
local AutoRoller = LibStub('AceAddon-3.0'):NewAddon('AutoRoller', 'AceEvent-3.0', 'AceConsole-3.0')
_G.AutoRoller = AutoRoller

local chatFrame = DEFAULT_CHAT_FRAME
local format = string.format

---------------   initialize   ---------------

-- {0 : pass, 1 : need, 2 : greed, 3 : nothing}

local defaults = {
	profile = {
		enabled = true,
		chat = true,
      disableBoPConfirm = false,
		rules = {
         {
            enabled = true,
            action = 2,
            logic = "|boe|"
         },  -- 1
      },
	},
}

AutoRoller.ruleNameLUT = {} --lookup table, for each new rule add [N],['rule'..N] = 'rule'..N, N

function AutoRoller:OnInitialize()
   self.db = LibStub('AceDB-3.0'):New('AutoRollerDB', defaults, 'Default')

   self.db.RegisterCallback(self, 'OnProfileChanged', 'OnEnable')
   self.db.RegisterCallback(self, 'OnProfileCopied', 'OnEnable')
   self.db.RegisterCallback(self, 'OnProfileReset', 'OnEnable')

	self:RegisterOptions()
end

function AutoRoller:OnEnable()
   local profile = self.db.profile
   if not profile.enabled then
      return self:Disable()
   end

   local lut,n = self.ruleNameLUT
   self.optionTable.args.rules.args = {}
   for i=1,#(profile.rules) do
      n='rule'..i
      lut[i],lut[n] = n,i
      self:BuildRuleOption(i, n)
   end

   self:RegisterEvent('CONFIRM_LOOT_ROLL')
   return self:RegisterEvent('START_LOOT_ROLL')
end

function AutoRoller:OnDisable()
end

---------------   Core   ---------------

local tags = {
   --hard coded values
   poor = 0,
   common = 1,
   uncommon = 2,
   rare = 3,
   epic = 4,
   legendary = 5,
   artifact = 6,
   heirloom = 7,

   --mutable per item
   raid = false, --(GetNumRaidMembers() > 0),
	party = true, --not raid
	combat = false, --InCombatLockdown()
	quality = 2,
	itype = "Armor", --"Armor", "Consumable", "Container", "Gem", "Key", "Miscellaneous", "Money", "Reagent", "Recipe", "Projectile", "Quest", "Quiver", "Trade Goods", "Weapon"
   subtype = "Plate", --http://www.wowwiki.com/ItemType
   ilvl = 213,
   reqlvl = 80,
   name = "Item of Lewting",
   id = 123456,
   boe = true,
   bop = false,
}

setmetatable(tags, {__index = _G})

local function GetItemID(link)
   local _,_,id = link:find("item:(.-):")
   return id
end

--return false, error
--       true, result
local function pcallLogic(logic)
   local func, error = loadstring('return (('..logic..') or false) and true') -- make sure return is true or false
   if not func then
      return false, error
   end
   setfenv(func, tags)
   return pcall(func)
end

local function EvaluateRules(rules)
   --loop thru rules
   local success, val
   for _,rule in ipairs(rules) do
      if rule.enabled then
         success, val = pcallLogic(rule.logic)
         if success and val then
            return rule.action
         end
      end
   end

   return 3 --do nothing for now
end

function AutoRoller:ValidateLogic(info, value)
   local success, error = pcallLogic(value)
   return success or error
end

local RollType = {[0] = "pass", [1] = "need", [2] = "greed", [3] = "nothing"}

function AutoRoller:START_LOOT_ROLL(event, arg1)
   local profile = self.db.profile
   if not profile.enabled then return end
		-- arg1 == rollID
   local texture, name, count, quality, isBoP = GetLootRollItemInfo(arg1)
   local link = GetLootRollItemLink(arg1)
   local _,_,quality,ilvl,reqlvl, itemType, subType = GetItemInfo(link) -- cant use name (item must be in inventory)
--~    local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iStackCount = GetItemInfo(link);

   -- propgate tags
   local raid = GetNumRaidMembers() > 0
   tags.raid = raid
	tags.party = not raid
	tags.combat = InCombatLockdown()
	tags.quality = quality
	tags.itype = itemType
   tags.subtype = subType
   tags.ilvl = ilvl
   tags.reqlvl = reqlvl
   tags.name = name
   tags.id = GetItemID(link)
   tags.boe = not isBoP
   tags.bop = isBoP

   --evaluate rules
   local roll = EvaluateRules(self.db.profile.rules)

   --roll
   if profile.chat then
      chatFrame:AddMessage(format("|cff00AA00AutoRoller: rolling|r %s |cff00AA00on|r %s", RollType[roll], link))
   end

   --RollOnLoot(arg1, {0 : pass, 1 : need, 2 : greed, 3 : nothing})
   if roll ~= 3 then
      return RollOnLoot(arg1, roll)
   end
end

function AutoRoller:CONFIRM_LOOT_ROLL(event, id, roll)
   if not self.db.profile.disableBoPConfirm then return end

   ConfirmLootRoll(id, roll)
   --hide popup?
   StaticPopup_Hide('CONFIRM_LOOT_ROLL')
end

local function EnabledColor(enabled)
   if enabled then
      return 1, 1, 1, 1, 1, 1
   else
      return .7, .7, .7, .7, .7, .7
   end
end

AutoRoller.dataobj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject('AutoLauncher', {
   type = 'launcher',
   icon = 'Interface\\Buttons\\UI-GroupLoot-Dice-Up',
   tocname = 'AutoRoller',
   label = 'AutoRoller',
   OnTooltipShow = function(tooltip)
      tooltip:AddLine("Auto	Roller")
      local profile = AutoRoller.db.profile
      local enabled = profile.enabled
      local red = enabled and 0 or 1
      tooltip:AddDoubleLine("Enabled", enabled and "Enabled" or "Disabled", 0, 1, 0, red, 1 - red, 0)
      enabled = profile.chat
      red = enabled and 0 or 1
      tooltip:AddDoubleLine("Chat", enabled and "Enabled" or "Disabled", 0, 1, 0, red, 1 - red, 0)
      for i,rule in pairs(profile.rules) do
         tooltip:AddDoubleLine(i..". "..rule.logic,  RollType[rule.action], EnabledColor(rule.enabled))
      end
   end,
})
